
def bincount(data, breaks):
    """Counts the number of elements of 'data' that lie within
    the intervals defined by 'breaks'.

    Specifically, for
    
    - data = [ d_1, d_2, ... , d_i, ..., d_n ]
    
    - breaks = [ b_1, b_2, ..., b_j, ..., b_m ]
    
    'bincount' returns a list l = [l_1, l_2,...,l_{m+1}] such that
    
    l_1     = #{ d_i |           d_i <= b_1 }
    l_2     = #{ d_i |     b_1 < d_i <= b_2 }
    ...
    l_m     = #{ d_i | b_{m-1} < d_i <= b_m }
    l_{m+1} = #{ d_i |     b_m < d_i         }
    
    """
    _b = list(breaks)
    _d = list(data)
    _b.sort()
    _d.sort()
    _b.append(max(_d) + 1)
    bins = []
    bidx = bcnt = didx = 0
    while didx < len(_d):
        if _d[didx] <= _b[bidx]:
            bcnt += 1
            didx += 1
        else:
            bidx += 1
            bins.append(bcnt)
            if bidx >= len(_b):
                break
            bcnt = 0
            while _d[didx] > _b[bidx] :
                bins.append( 0 )
                bidx += 1
    bins.append(bcnt)
    bins.extend( [0 for i in range(bidx + 1,len(_b))] )
    return bins

def histcount(data, nbins):
    """Returns the histogram count for 'data' with 'nbins' uniform bins.
    """
    m = min(data)
    M = max(data)
    f = lambda x: float(x) / nbins * (M - m) + m
    r = map(f,range(0,nbins))
    return bincount(data, r[1:])# semantics of bincount imply the
                                # min must *not* be passed

