from wxPython.wx import *
from Thuban.Model.histogram import histcount, bincount

class Graph(wxPanel):

    def __init__(self, parent, width, height):
        wxPanel.__init__(self, parent, -1, wxPoint(-1,-1), wxSize(width,height))
        self.w = width
        self.h = height
        self.flipx = lambda x: x
        self.flipy = lambda y: 1.0-y
        self.border = 0.02
        EVT_PAINT(self, self.OnPaint)
        
    def scaleit(self,x,y):
        """ From cartesian (x,y) normalized to [0,1], calculate screen coordinates. """
        return (self.w*((1.0-2.0*self.border)*self.flipx(x)+self.border),
                self.h*((1.0-2.0*self.border)*(self.flipy(y))+self.border))
    
    def draw_rect(self, dc, ul, lr):
        ulx,uly = self.scaleit(ul[0],ul[1])
        lrx,lry = self.scaleit(lr[0],lr[1])
        dc.DrawLine( ulx,uly,lrx,uly )
        dc.DrawLine( lrx,uly,lrx,lry )
        dc.DrawLine( lrx,lry,ulx,lry )
        dc.DrawLine( ulx,lry,ulx,uly )
        
    def draw_line(self,dc,x1,y1,x2,y2):
        x1,y1 = self.scaleit(x1,y1)
        x2,y2 = self.scaleit(x2,y2)
        dc.DrawLine(x1,y1,x2,y2)
        
    def OnPaint(self, event):
        dc = wxPaintDC(self)
        dc.Clear()
        (self.w, self.h) = dc.GetSizeTuple()
        self.refresh(dc)
        
    def refresh(self,dc):
        pass


class PointDensityGram(Graph):

    def __init__(self, parent, data=None):
        Graph.__init__(self, parent,400,20)
        if len(data)>0:
            mxd = max(data)
            mnd = min(data)
            self.data = [ (d-mnd)/(mxd-mnd) for d in data ]
        else:
            self.data = [data]

    def refresh(self, dc):
        dc.SetPen(wxPen(wxBLACK,1))
        self.draw_rect(dc, (0.0,0.0),(1.0,1.0))
        for d in self.data:
            self.draw_line(dc, d,0.0,d,1.0 )

class Histogram(Graph):

    def __init__(self, parent, data=None, bins=10, flipy=0):
        Graph.__init__(self, parent, 400, 200)
        self.data = data
        if type(bins) == type([]):
            self.bins = bins
            self.nbins = None
        else:
            self.nbins = bins
            self.bins = None
        if flipy:
            self.flipy = lambda x: x

    def draw_hist(self,dc, values, series, binsizes=None):
        mxs = max(series)
        y = [ float(v)/mxs for v in series ]
        
        mnv = float(min(values))
        mxv = float(max(values))
        if binsizes is None:
            x = [ (float(v)-mnv)/(mxv-mnv) for v in values ]
            x.sort()
            dx = filter( lambda x: x>0 ,
                         [ x[i+1]-x[i] for i in range(len(x)-1) ]  )
            binsizes = [ min(dx) ] * len(x)
        x = [ float(values[i]-mnv)/(mxv-mnv) # values normalized to [0,1]
              for i in range(len(x)) ]
        mxx = max(x)
        binsizes = [ 0.99*b for b in binsizes ]
        dc.SetPen(wxPen(wxNamedColor("black"),1))
        self.draw_rect(dc, (0.0,0.0), (1.0,1.0) )
        dc.SetPen(wxPen(wxNamedColor("blue"),2))
        for i in range(len(y)):
            self.draw_rect(dc, 
                           (max(x[i]-0.5*binsizes[i],0),y[i]),
                           (min(x[i]+0.5*binsizes[i],mxx),0.0) )
            xb = x[i]
            self.draw_line(dc,xb,0.0,xb,.02 )
            
    def refresh(self,dc):
        self.draw_hist(dc)


class BinHistogram(Histogram):
    def refresh(self,dc):
        y = histcount(self.data, self.nbins)
        x = [ float(v)/self.nbins for v in range(self.nbins) ]
        self.draw_hist(dc, x, y)

class ClassHistogram(Histogram):
    def refresh(self,dc):
        y = bincount(self.data, self.bins)
        x = self.data
        self.draw_hist(dc, x, y)



